import time

# noinspection PyPackageRequirements
import discord
from aiohttp import web
from nacl.exceptions import BadSignatureError
from v6d0auth import certs


def define_routes(routes: web.RouteTableDef, client: discord.Client):
    @routes.get('/')
    async def home(_request: web.Request):
        return web.Response(body='v6d3losyash\n')

    @routes.post('/stop')
    async def stop(request: web.Request):
        try:
            assert abs(float(certs.verify(await request.read())) - time.time()) < 1
        except ValueError:
            raise web.HTTPBadRequest
        except BadSignatureError:
            raise web.HTTPUnauthorized
        except AssertionError:
            raise web.HTTPRequestTimeout
        else:
            await client.change_presence(status=discord.Status.offline)
            await client.close()
            raise web.HTTPOk


def app_routes(client: discord.Client) -> web.RouteTableDef:
    routes = web.RouteTableDef()
    define_routes(routes, client)
    return routes


def app_with_routes(routes: web.RouteTableDef):
    app = web.Application()
    app.add_routes(routes)
    return app


def get_app(client: discord.Client) -> web.Application:
    return app_with_routes(app_routes(client))
